grammar Gramatica;

options
{
	output = AST;
	ASTLabelType = CommonTree;	
}

tokens
{		
	ABREPAR = '(';
	FECHAPAR = ')';
	VIRGULA = ',';
	
	MULT = '*';
	DIV = '/';
	MAIS = '+';
	MENOS = '-';	
	IGUAL = '=';
	DIFERENTE1 = '<>'; 
	DIFERENTE2 = '><';
	MAIORIGUAL = '>=';
	MENORIGUAL = '<=';
	MAIORQUE = '>';
	MENORQUE = '<';
	
	Programa;
	Programa_Declaracao;
	Programa_Metodo;
	Lista_Instrucoes;
	
	Declaracao;
	Atribuicao;
	Variavel;
	ListaVariavel; 
	Tipo;	 
	ListaParametros; 
	Funcao;	
	Sub; 
	Exit; 	
	Expressao;
	ListaDeExpressao;
	FatorBooleano; 
	Comparacao;
	Inequacao;
	Termo;
	Fator;
	ExpressaoUnaria;
	Unidade;
	Constante_NUM;
	Constante_BOOL;
	Constante_STRING; 
	Chamada;	
	Instrucoes;
	Operador_NOT;
	Operadores_Termo;
	Operadores_Fator;
	Operadores_Comparacao;
	Operadores_Inequacao;
	Instrucao_IF;
	Bloco_If;
	Bloco_Else;
	Instrucao_While;
	Instrucao_Until;
	Instrucao_Input;
	Instrucao_Print;
	Instrucao_For;
	Constante;
						
}

//Opoes de sada
@lexer::header { package output; }
@parser::header { package output; }

// Palvras reservadas
//Tipos
  BOOLEANO: B O O L E A N;
  INTEIRO	: I N T E G E R;
  STRING	: S T R I N G;
  
//Comandos
   PROGRAM 	:	 P R O G R A M;
   IF		:	 I F;
   THEN		:	 T H E N;
   ELSE		:	 E L S E;
   FOR		:	 F O R;
   TO		:	 T O;
   STEP 	:	 S T E P;
   NEXT 	:	 N E X T;
   EXIT 	:	 E X I T;
   DO 		:	 D O;
   WHILE	:	 W H I L E;
   LOOP 	:	 L O O P;
   UNTIL	:	 U N T I L;   
   PRINT 	: 	P R I N T;
   INPUT 	: 	I N P U T;     	

   FUNCTION	: 	F U N C T I O N;
   AS		: 	A S;
   END		: 	E N D;
   SUB		: 	S U B;
	

   NOT 		: N O T;
   OR		: O R;
   AND		: A N D;	

   
  

//Alfabeto
fragment A	:	'a' | 'A';
fragment B	:	'b' | 'B';
fragment C	:	'c' | 'C';
fragment D	:	'd' | 'D';
fragment E	:	'e' | 'E';
fragment F	:	'f' | 'F';
fragment G	:	'g' | 'G';
fragment H	:	'h' | 'H';
fragment I	:	'i' | 'I';
fragment J	:	'j' | 'J';
fragment K	:	'k' | 'K';
fragment L	:	'l' | 'L';
fragment M	:	'm' | 'M';
fragment N	:	'n' | 'N';
fragment O	:	'o' | 'O';
fragment P	:	'p' | 'P';
fragment Q	:	'q' | 'Q';
fragment R	:	'r' | 'R';
fragment S	:	's' | 'S';
fragment T	:	't' | 'T';
fragment U	:	'u' | 'U';
fragment V	:	'v' | 'V';
fragment W	:	'w' | 'W';
fragment X	:	'x' | 'X';
fragment Y	:	'y' | 'Y';
fragment Z	:	'z' | 'Z';


programa 	
	: 	PROGRAM ID (declaracao)* (metodo)* (instrucoes1)* END EOF -> ^(Programa ID (declaracao)* (metodo)* (instrucoes1)*)
	;

//corpoPrograma	
//	:	(declaracao  -> ^(Programa_Declaracao declaracao )
//		|metodo -> ^(Programa_Metodo metodo))
//		|instrucoes1 -> ^(Lista_Instrucoes instrucoes1)
//	;
	
input
	:	INPUT listaVariavel -> ^(Instrucao_Input listaVariavel)
	;
	
print
	:	PRINT listaDeExpressao -> ^(Instrucao_Print listaDeExpressao)
	;

declaracao 
	:	tipo listaVariavel -> ^(Declaracao tipo listaVariavel)
	;

metodo	
	:	(metodoFunction -> ^(metodoFunction)
		| metodoSub -> ^(metodoSub))
	;
	
listaVariavel 
	:	 ID (VIRGULA ID)* -> ^(ListaVariavel ID (ID)*); 
	
tipo	
	:	INTEIRO -> ^(Tipo INTEIRO) 
		| BOOLEANO -> ^(Tipo BOOLEANO)
		| STRING -> ^(Tipo STRING)
	; 	 

listaParametros 
	:	(ID AS tipo)(VIRGULA ID AS tipo)* -> ^(ListaParametros ID tipo(ID tipo)*)
	;

metodoFunction	
	:	FUNCTION ID ABREPAR listaParametros? FECHAPAR AS tipo (declaracao)* (instrucoes1)*  END FUNCTION -> ^(Funcao ID listaParametros? tipo (declaracao)* (instrucoes1)*)
	;

metodoSub 
	:	SUB ID ABREPAR listaParametros? FECHAPAR (declaracao)* (instrucoes1)* END SUB -> ^(Sub ID listaParametros? (declaracao)* (instrucoes1)*)
	;

	
instrucoes1	
	:	instrucoes2 -> ^(Instrucoes instrucoes2)
	;
	
/*
		
	O que foi feito (Classes):
	
	blocoIf
	blocoElse
	Instrues
	InstrucaoIf
	InstrucaoWhile
	InstrucaoUntil	
	Funcao
	Sub
	ListaExpressao
	Atribuicao		
		
	Expressao
	FatorBooleano
	Comparacao
	Inequacao
	OperadorComparacao	
	TermoNumerico
	OperadorInequacao 
	FatorNumerico
	OperadorTermoNumerico
	ExpressaoUnaria
	OperadorFatorNumerico
	Unidade <----------------------
	Constante
	

	
		
*/
instrucoes2	
	:	DO WHILE expressao (instrucoes1)* LOOP -> ^(Instrucao_While expressao (instrucoes1)*)//
		 | IF expressao THEN blocoIf ELSE blocoElse END IF -> ^(Instrucao_IF expressao blocoIf blocoElse)//
		 | DO (instrucoes1)* LOOP UNTIL expressao -> ^(Instrucao_Until (instrucoes1)* expressao)//
		 | FOR ID IGUAL expressao TO expressao (STEP expressao)? (instrucoes1)* NEXT ID -> ^(Instrucao_For ID expressao expressao (expressao)? (instrucoes1)* ID)
		 | atribuicao -> ^(atribuicao)
		 | input -> ^(input)
		 | print -> ^(print) 
		 | chamada -> ^(chamada)
		 | exit -> ^(exit)	
	;

blocoIf	
	: (instrucoes1)* -> ^(Bloco_If (instrucoes1)*)
	;

blocoElse 
	: (instrucoes1)* -> ^(Bloco_Else (instrucoes1)*)
	;
	
exit 	
	:	EXIT FOR -> ^(EXIT FOR) 
		| EXIT DO -> ^(EXIT DO)
	;

expressao 
	:	fatorBooleano (OR fatorBooleano)* -> ^(Expressao fatorBooleano (fatorBooleano)*)
	;	
	
fatorBooleano 
	:	inequacao(AND inequacao)* -> ^(FatorBooleano inequacao (inequacao)*)
	;
	
inequacao
	:	comparacao(operadoresInequacao comparacao)? -> ^(Inequacao comparacao(operadoresInequacao comparacao)?)
	;
	
comparacao
	:	termo(operadoresComparacao termo)? -> ^(Comparacao termo(operadoresComparacao termo)?)
	;

termo	
	:	fator (operadoresTermo fator)* -> ^(Termo fator(operadoresTermo fator)*)
	;

fator 	
	:	expressaoUnaria (operadoresFator expressaoUnaria)* -> ^(Fator expressaoUnaria (operadoresFator expressaoUnaria)*)
	;

expressaoUnaria 
	:	(NOT)? unidade -> ^(ExpressaoUnaria (NOT)? unidade)	
	;

unidade :	unidade2 -> ^(Unidade unidade2)
	;

unidade2	
	:	ABREPAR expressao FECHAPAR -> ^(expressao)
		| constante -> ^(Constante constante)
		| ID -> ^(Variavel ID)
		| chamada -> ^(chamada)	
	;

constante 
	:	VALNUM -> ^(Constante_NUM)
		|VALBOOL -> ^(Constante_BOOL)
		|VALSTRING -> ^(Constante_STRING)	
	;
	
operadoresComparacao
	:	 IGUAL -> ^(Operadores_Comparacao)
		 | DIFERENTE1 -> ^(Operadores_Comparacao) 
		 | DIFERENTE2 -> ^(Operadores_Comparacao)
	;
	
operadoresInequacao
	:	MAIORIGUAL -> ^(Operadores_Inequacao)
		| MENORIGUAL -> ^(Operadores_Inequacao) 
		| MAIORQUE -> ^(Operadores_Inequacao)
		| MENORQUE -> ^(Operadores_Inequacao)
	;


operadoresTermo
	:	(MAIS -> ^(Operadores_Termo) 
		| MENOS -> ^(Operadores_Termo))
	;
	
operadoresFator 
	:	(MULT -> ^(Operadores_Fator)
		| DIV -> ^(Operadores_Fator))
	;	

	
listaDeExpressao
	:	expressao (VIRGULA expressao)* -> ^(ListaDeExpressao expressao (expressao)*)
	;
	
atribuicao
	:	ID IGUAL expressao -> ^(Atribuicao ID expressao)
	;
 
chamada	
	:	ID ABREPAR (listaDeExpressao)? FECHAPAR -> ^(Chamada ID (listaDeExpressao)?)	
	;


VALSTRING
	:	'\"' (' ' | '!' | ('#'..'~'))* '\"';

VALBOOL	:	'TRUE'| 'FALSE'|'true'|'false'|'True'|'False'|'TRue'|'FAlse';

ID	: 	('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'0'..'9')*;

VALNUM	:	 ('-')?( '0'..'9')+;

WS:	('\t' | ' ' | '\r' | '\n'| '\u000C')+ { $channel = HIDDEN; };
