package adaptors;

import org.antlr.runtime.Token;
import org.antlr.runtime.tree.CommonTreeAdaptor;

import com.sun.org.apache.bcel.internal.generic.NEW;

import terminais.BooleanType;
import terminais.Identificador;
import terminais.IntegerType;
import terminais.StringType;

import antlr.GramaticaLexer;
import ast.Atribuicao;
import ast.Bloco_Else;
import ast.Bloco_If;
import ast.Chamada;
import ast.Comparacao;
import ast.Constante;
import ast.Declaracao;
import ast.Expressao;
import ast.ExpressaoUnaria;
import ast.FatorBooleano;
import ast.FatorNumerico;
import ast.Funcao;
import ast.Inequacao;
import ast.InstrucaoFor;
import ast.InstrucaoIf;
import ast.InstrucaoInput;
import ast.InstrucaoPrint;
import ast.InstrucaoUntil;
import ast.InstrucaoWhile;
import ast.Instrucoes;
import ast.ListaExpressoes;
import ast.ListaParamentros;
import ast.ListaVariavel;
import ast.OperadorFatorNumerico;
import ast.OperadorTermoNumerico;
import ast.OperadoresInequacao;
import ast.OperadoresComparacao;
import ast.Programa;
import ast.Sub;
import ast.TermoNumerico;
import ast.Tipo;
import ast.Unidade;
import ast.Variavel;

public class GramaticaTreeAdaptor extends CommonTreeAdaptor {

	public Object create(Token payload) {

		if (payload != null) {
            switch (payload.getType()) {
            
            case GramaticaLexer.Programa:
            	return new Programa(payload);
            case GramaticaLexer.ID:
            	return new Identificador(payload);
            case GramaticaLexer.Declaracao:
            	return new Declaracao(payload);
            case GramaticaLexer.ListaVariavel:
            	return new ListaVariavel(payload);
            case GramaticaLexer.ListaParametros:
            	return new ListaParamentros(payload);
            case GramaticaLexer.Tipo:
            	return new Tipo(payload);
            case GramaticaLexer.INTEIRO:
            	return new IntegerType(payload);
            case GramaticaLexer.BOOLEANO:
            	return new BooleanType(payload);
            case GramaticaLexer.STRING:
            	return new StringType(payload);
                     	
            case GramaticaLexer.Bloco_Else:
            	return new Bloco_Else(payload);   
            case GramaticaLexer.Bloco_If:
            	return new Bloco_If(payload);                	
            case GramaticaLexer.Instrucoes:
            	return new Instrucoes(payload);                	
            case GramaticaLexer.Instrucao_For:
            	return new InstrucaoFor(payload);
            case GramaticaLexer.Instrucao_While:
            	return new InstrucaoWhile(payload);
            case GramaticaLexer.Instrucao_IF:
            	return new InstrucaoIf(payload);
            case GramaticaLexer.Instrucao_Until:
            	return new InstrucaoUntil(payload);
            case GramaticaLexer.Atribuicao:
            	return new Atribuicao(payload);
            case GramaticaLexer.Instrucao_Input:
            	return new InstrucaoInput(payload);
            case GramaticaLexer.Instrucao_Print:
            	return new InstrucaoPrint(payload);
            case GramaticaLexer.Chamada:
            	return new Chamada(payload);
            case GramaticaLexer.Expressao:
            	return new Expressao(payload);
            case GramaticaLexer.FatorBooleano:
            	return new FatorBooleano(payload);
            case GramaticaLexer.Comparacao:
            	return new Comparacao(payload);
            case GramaticaLexer.Inequacao:
            	return new Inequacao(payload);
            case GramaticaLexer.Operadores_Comparacao:
            	return new OperadoresComparacao(payload);
            case GramaticaLexer.Termo:
            	return new TermoNumerico(payload);
            case GramaticaLexer.Operadores_Inequacao:
            	return new OperadoresInequacao(payload);
            case GramaticaLexer.Fator:
            	return new FatorNumerico(payload);
            case GramaticaLexer.Operadores_Termo:
            	return new OperadorTermoNumerico(payload);
            case GramaticaLexer.ExpressaoUnaria:
            	return new ExpressaoUnaria(payload);
            case GramaticaLexer.Operadores_Fator:
            	return new OperadorFatorNumerico(payload);
            case GramaticaLexer.Unidade:
            	return new Unidade(payload);
            case GramaticaLexer.Constante:
            	return new Constante(payload);
            case GramaticaLexer.Funcao:
            	return new Funcao(payload);
            case GramaticaLexer.Sub:
            	return new Sub(payload);
            case GramaticaLexer.ListaDeExpressao:
            	return new ListaExpressoes(payload);
            case GramaticaLexer.Constante_NUM:
            	return new IntegerType(payload);
            case GramaticaLexer.Constante_BOOL:
            	return new BooleanType(payload);
            case GramaticaLexer.Constante_STRING:
            	return new StringType(payload);
            case GramaticaLexer.Variavel:
            	return new Variavel(payload);
            	
            
            }            
            
            
		}		
		  return super.create(payload);
		
	}
	
}
