package antlr;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import terminais.BooleanType;
import terminais.Identificador;
import terminais.IntegerType;
import terminais.StringType;
import terminais.TiposPrimitivos;
import ast.Atribuicao;
import ast.Bloco_Else;
import ast.Bloco_If;
import ast.Chamada;
import ast.Comparacao;
import ast.Constante;
import ast.Declaracao;
import ast.Expressao;
import ast.ExpressaoUnaria;
import ast.Expression;
import ast.FatorBooleano;
import ast.FatorNumerico;
import ast.Funcao;
import ast.Inequacao;
import ast.InstrucaoFor;
import ast.InstrucaoIf;
import ast.InstrucaoInput;
import ast.InstrucaoPrint;
import ast.InstrucaoUntil;
import ast.InstrucaoWhile;
import ast.Instrucoes;
import ast.ListaExpressoes;
import ast.ListaParamentros;
import ast.ListaVariavel;
import ast.OperadorFatorNumerico;
import ast.OperadorTermoNumerico;
import ast.OperadoresComparacao;
import ast.OperadoresInequacao;
import ast.Parametro;
import ast.Programa;
import ast.Sub;
import ast.TermoNumerico;
import ast.Tipo;
import ast.Unidade;
import ast.Variavel;
import terminais.Terminal;
import framework.CompilationEnvironment;
import framework.ConsoleErrorReporter;
import framework.IVisitor;

public class GramaticaChecker implements IVisitor{

	private ConsoleErrorReporter errorReporter;
	private CompilationEnvironment environment;

	public GramaticaChecker (ConsoleErrorReporter errorReporter){
		this.errorReporter = errorReporter;
		this.environment = new CompilationEnvironment();
	}	

	@Override
	public Object visit(Programa program, Object obj) {
		ArrayList<Declaracao> declaracoes = program.getDeclaracoes();

		for (int i = 0; i < declaracoes.size(); i++){
			Declaracao declaracao = declaracoes.get(i);
			declaracao.accept(this, obj);
		}		

		ArrayList<Funcao> funcoes = program.getFuncoes();
		for (int i = 0; i < funcoes.size(); i++){
			Funcao func = funcoes.get(i);
			func.accept(this, obj);
		}	
		
		ArrayList<Sub> subs = program.getSub();
		for (int i = 0; i < subs.size(); i++){
			Sub s = subs.get(i);
			s.accept(this, obj);
		}	

		ArrayList<Instrucoes> intrucoes = program.getInstrucoes();
		for (int i = 0; i < intrucoes.size(); i++){
			Instrucoes intrucao = intrucoes.get(i);
			intrucao.accept(this, obj);
		}				
		
		return null;
	}


	@Override
	public Object visit(StringType stringType, Object obj) {
		return TiposPrimitivos.String;
	}

	@Override
	public Object visit(IntegerType intergerType, Object obj) {		
		return TiposPrimitivos.Integer;
	}

	@Override
	public Object visit(BooleanType booleanType, Object obj) {
		return TiposPrimitivos.Boolean;
	}


	@Override
	public Object visit(Identificador id, Object obj) {
		if (environment.containsIdentifier(id))
			return environment.getTipoID(id);
		else {
			errorReporter.reportError(id, "identificador no declarado: " + id.getValor() + " -- visit (Identificador)");
			System.exit(0);
			return null;
		}
	}

	@Override
	public Object visit(ListaVariavel listaVariavel, Object obj) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Object visit(Tipo tipo, Object o) {		
		return tipo.accept(this, o);
	}


	@Override
	public Object visit(Bloco_If blocoIf, Object obj) {
		List instrucoes = blocoIf.getInstrucoes();

		for (int i = 0; i < instrucoes.size(); i++){
			Instrucoes instrucao = (Instrucoes)instrucoes.get(i);
			instrucao.accept(this, null);
		}
		return null;
	}

	@Override
	public Object visit(InstrucaoFor instrucaoFor, Object obj) {
		
		System.out.println("instruo for");
		//Validando a atribuio
		Atribuicao atr = instrucaoFor.getAtribuicao();
		atr.accept(this, obj);
		
		Expressao exp = atr.getExpressao();
		TiposPrimitivos tp = (TiposPrimitivos)exp.accept(this, obj);
		
		if (tp!=TiposPrimitivos.Integer){
			errorReporter.reportError(instrucaoFor, "Erro na atribuio do for");				
			System.exit(0);		
		}
		
		//Valdando a condicao de parado do For
		Expressao exp2 = instrucaoFor.getExpressaoParada();
		TiposPrimitivos tp2 = (TiposPrimitivos)exp2.accept(this, obj);
		
		if (tp2!=TiposPrimitivos.Integer){
			errorReporter.reportError(instrucaoFor, "Erro na expresso de parada do for");				
			System.exit(0);		
		}
		
		//Validando o Step
		if (instrucaoFor.getExpressaoOpcional()!=null){
		
			Expressao expOpcional = instrucaoFor.getExpressaoOpcional();
			TiposPrimitivos tipoExpOpc = (TiposPrimitivos)expOpcional.accept(this, obj);
			
			if (tipoExpOpc!=TiposPrimitivos.Integer){
				errorReporter.reportError(instrucaoFor, "O Step do for tem que ser um inteiro");	
				System.exit(0);		
			}
		}	
		
		return null;
	}

	@Override
	public Object visit(Instrucoes instrucoes, Object obj) {
		Object inst = instrucoes.getChild(0);
		if (inst instanceof Atribuicao){
			Atribuicao atr = (Atribuicao) inst;
			atr.accept(this, obj);
		}	else if (inst instanceof InstrucaoInput){
			InstrucaoInput input = (InstrucaoInput) inst;
			input.accept(this, obj);
		}	else if (inst instanceof Chamada){			
			Chamada chamada = (Chamada) inst;
			chamada.accept(this, obj);
		}	else if (inst instanceof InstrucaoIf){
			InstrucaoIf instIf = (InstrucaoIf) inst;
			instIf.accept(this, obj);
		}	else if (inst instanceof InstrucaoPrint){
			InstrucaoPrint instPRint = (InstrucaoPrint) inst;
			instPRint.accept(this, obj);			
		}	else if (inst instanceof InstrucaoWhile){
			InstrucaoWhile instWhile = (InstrucaoWhile) inst;
			instWhile.accept(this, obj);			
		}	else if (inst instanceof InstrucaoUntil){
			InstrucaoUntil instUntil = (InstrucaoUntil) inst;
			instUntil.accept(this, obj);			
		}	else if (inst instanceof InstrucaoFor){
			InstrucaoFor instFor = (InstrucaoFor) inst;
			instFor.accept(this, obj);			
		}	

		return null;
	}

	@Override
	public Object visit(Bloco_Else blocoElse, Object obj) {				
		List instrucoes = blocoElse.getInstrucoes();

		for (int i = 0; i < instrucoes.size(); i++){
			Instrucoes instrucao = (Instrucoes)instrucoes.get(i);
			instrucao.accept(this, null);
		}

		return null;
	}

	@Override
	public Object visit(InstrucaoIf instrucaoIf, Object obj) {
		System.out.println("Instrucao if");
		Expressao expBool = instrucaoIf.getCondicao();

		TiposPrimitivos tipoExp = (TiposPrimitivos) expBool.accept(this, obj);

		if (tipoExp == TiposPrimitivos.Boolean){
			instrucaoIf.getBlocoIf().accept(this, obj);
			instrucaoIf.getBlocoElse().accept(this, obj);
		} else{
			errorReporter.reportError(instrucaoIf, "A instruo condicional do if deve conter uma expresso booleana");				
			System.exit(0);			
		}


		return null;
	}

	@Override
	public Object visit(Expressao expressao, Object o) {		

		ArrayList<FatorBooleano> lista = expressao.getFatorBoleano();

		if(lista.size() == 1){
			return lista.get(0).accept(this, o);
		}else{
			for(int i = 0; i<lista.size(); i++){
				FatorBooleano exp = lista.get(i);
				exp.accept(this, o);	
			}
			return TiposPrimitivos.Boolean;
		}
	}

	@Override
	public Object visit(FatorBooleano fatorBooleano, Object o) {
		//System.out.println("Fator Booleano");
		ArrayList<Inequacao> lista = fatorBooleano.getInequacao();		

		if(lista.size()==1){
			return lista.get(0).accept(this, o);
		}else{

			for(int i = 0; i < lista.size(); i++){
				Inequacao comp = lista.get(i);
				comp.accept(this,o);
			}
			return TiposPrimitivos.Boolean;
		}		
	}

	@Override
	public Object visit(Comparacao comparacao, Object o) {
		TermoNumerico termN = comparacao.getTermoNumerico();
		OperadoresComparacao opComp = comparacao.getOperadorComparacao();
		TermoNumerico termN2 = comparacao.getTermoNumerico2();

		if(opComp != null){
			if(termN2 != null){
				opComp.accept(this,o);
				termN2.accept(this,o);

				return TiposPrimitivos.Boolean;
			}else{
				return null;
			}
		}else{
			return termN.accept(this, o);			
		}
	}

	@Override
	public Object visit(Inequacao inequacao, Object o) {
		Comparacao comp = inequacao.getComparacao();
		OperadoresInequacao opComp = inequacao.getOperadorInequacao();
		Comparacao comp2 = inequacao.getComparacao2();

		//termNum.accept(this,o);

		if(opComp != null){
			if(comp2 != null){
				opComp.accept(this,o);
				comp2.accept(this,o);
				return TiposPrimitivos.Boolean;
			}else{
				return null;
			}
		}else{
			return comp.accept(this, o);			
		}		

	}

	@Override
	public Object visit(TermoNumerico termoNumerico, Object o) {
		FatorNumerico fatNum1 = termoNumerico.getFatorNumerico();
		ArrayList<ArrayList<Object>> filhos = termoNumerico.getFilhos();
		if(filhos == null || filhos.size() == 0){
			return fatNum1.accept(this, o);	
		}else{
			TiposPrimitivos retorno = (TiposPrimitivos) fatNum1.accept(this, o);

			for(int i = 0; i < filhos.size(); i++){
				//OperadorTermoNumerico op = (OperadorTermoNumerico) filhos.get(i).get(0);
				FatorNumerico ft = (FatorNumerico) filhos.get(i).get(1);

				if (retorno != (TiposPrimitivos) ft.accept(this, o)) {
					return TiposPrimitivos.Unknow;
				}

			}			
			return retorno;
		}

	}

	@Override
	public Object visit(OperadoresComparacao operadorComparacao, Object o) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Object visit(FatorNumerico fatorNumerico, Object o) {
		ExpressaoUnaria expUn = fatorNumerico.getExpressaoUnaria1();

		ArrayList<ArrayList<Object>> filhos = fatorNumerico.getFilhos();
		if(filhos == null || filhos.size() == 0){
			return expUn.accept(this, o);	
		}else{
			expUn.accept(this, o);

			for(int i = 0; i < filhos.size(); i++){
				OperadorFatorNumerico op = (OperadorFatorNumerico) filhos.get(i).get(0);
				op.accept(this, o);
				ExpressaoUnaria fat = (ExpressaoUnaria) filhos.get(i).get(1);
				fat.accept(this, o);
			}			
			return TiposPrimitivos.Integer;
		}
	}

	@Override
	public Object visit(ExpressaoUnaria expressaoUnaria, Object o) {
		Unidade unid = expressaoUnaria.getUnidade();
		return unid.accept(this, o);
	}

	@Override
	public Object visit(OperadorFatorNumerico operadorFatorNumerico, Object o) {
		return null;
	}

	@Override
	public Object visit(OperadoresInequacao operadorInequacao, Object o) {
		return TiposPrimitivos.Unknow;
	}

	@Override
	public Object visit(OperadorTermoNumerico operadorTermoNumerico, Object o) {
		return null;
	}

	@Override
	public Object visit(Unidade unidade, Object o) {
		Expressao exp = unidade.getExpressao();
		Constante con = unidade.getConstante();
		Identificador id = unidade.getId();
		Chamada cham = unidade.getChamada();

		if(exp != null){
			return exp.accept(this, o);
		}else if(con != null){
			return con.accept(this,o);
		}else if(id != null){
			return id.accept(this,o);
		}else if(cham != null){
			return cham.accept(this,o);
		}else{
			return TiposPrimitivos.Unknow;
		}
	}

	@Override
	public Object visit(Constante constante, Object o) {
		BooleanType boolit = constante.getValorBooleano();
		IntegerType intlit = constante.getValorNumerico();
		StringType string = constante.getValorString();

		if(boolit != null){			
			return boolit.accept(this, o);
		}else if(intlit != null){
			return intlit.accept(this, o);
		}else if(string != null){
			return string.accept(this,o);
		}else{
			//System.out.println("problema em Constante - AulaChecker.java");
		}
		return TiposPrimitivos.Unknow;
	}

	@Override
	public Object visit(InstrucaoWhile instrucaoWhile, Object o) {

		System.out.println("Visit da instrucao While");
		List<Instrucoes> instrucoes = instrucaoWhile.getInstrucoes();		
		Expressao expBool = instrucaoWhile.getCondicao();
		
		TiposPrimitivos tipoExp = (TiposPrimitivos) expBool.accept(this, o);
				
		System.out.println("Tipo da expresso do visit(while): " + tipoExp.toString());
		System.out.println(TiposPrimitivos.Boolean);
		
		if (tipoExp.toString() == TiposPrimitivos.Boolean.toString()){
			
			System.out.println("entrei");
			for (int i = 0; i < instrucoes.size(); i++){
				Instrucoes instrucao = (Instrucoes)instrucoes.get(i);
				instrucao.accept(this, null);
			}
			
			
		} else{
			errorReporter.reportError(instrucaoWhile, "A instruo condicional do While deve conter uma expresso booleana");				
			System.exit(0);			
		}
		
		return null;
	}

	@Override
	public Object visit(InstrucaoUntil instrucaoUntil, Object o) {
		
		System.out.println("Instrucao Until");
		Expressao expBool = instrucaoUntil.getCondicao();
		
		TiposPrimitivos tipoExp = (TiposPrimitivos) expBool.accept(this, o);
		List<Instrucoes> instrucoes = instrucaoUntil.getInstrucoes();
		
		if (tipoExp == TiposPrimitivos.Boolean){
			
			for (int i = 0; i < instrucoes.size(); i++){
				Instrucoes instrucao = (Instrucoes)instrucoes.get(i);
				instrucao.accept(this, null);
			}
			
			
		} else{
			errorReporter.reportError(instrucaoUntil, "A instruo condicional do Loop Until deve conter uma expresso booleana");				
			System.exit(0);			
		}
		
		return null;
	}

	@Override
	public Object visit(ListaParamentros listaParamentros, Object o) {
		Vector<Parametro> lista = listaParamentros.getParametros();

		for (int i = 0; i < lista.size(); i++){
			Parametro param = lista.get(i);

			Tipo tipo = (Tipo) param.getTipo();
			TiposPrimitivos tipoTipo = (TiposPrimitivos) tipo.accept(this, o);

			Identificador idTemp = param.getId(); 

			idTemp.accept(this, o);
			if(!environment.containsIdentifier(idTemp)){
				environment.addIdentificador(idTemp, tipoTipo);				
			}else{
				errorReporter.reportError(idTemp, "identificador ja utilizado");
			}
		}
		return null;
	}

	@Override
	//public Object visit(Funcao funcao, Map<String, TiposPrimitivos> mapping, Object obj) {
	public Object visit(Funcao funcao, Object o) {
		//Map<String, TiposPrimitivos> map = mapping;		

		System.out.println("Visit de funo");
		ListaParamentros listaPar = funcao.getListaParametros();
		Vector<Parametro> lista = listaPar.getParametros();

		for (int i = 0; i < lista.size(); i++){
			Parametro param = lista.get(i);

			Tipo tipo = (Tipo) param.getTipo();
			TiposPrimitivos tipoTipo = tipo.getTipo();
			Identificador idTemp = param.getId();

			if(!environment.containsIdentifier(idTemp)){
				environment.addIdentificador(idTemp, tipoTipo);					
			}else{
				errorReporter.reportError(idTemp, "identificador ja utilizado!");
				System.exit(0);
			}
		}

		ArrayList<Declaracao> declaracoes = funcao.getDeclaracoes();		
		for (int i = 0; i < declaracoes.size(); i++){
			Declaracao decl = declaracoes.get(i);

			Tipo tipo = (Tipo) decl.getTipo();
			TiposPrimitivos tipoTipo = tipo.getTipo();
			//TiposPrimitivos tipoTipo = (TiposPrimitivos) tipo.accept(this, o);

			ListaVariavel listVar = decl.getListaVariavel();
			Vector<Identificador> listaIdent = listVar.getIdentificadores(); 

			for (int j=0; j<listaIdent.size(); j++){
				Identificador idTemp = listaIdent.elementAt(j);
				idTemp.accept(this, o);

				if(!environment.containsIdentifier(idTemp)){					
					environment.addIdentificador(idTemp, tipoTipo);					
				}else{
					errorReporter.reportError(idTemp, "identificador ja utilizado!");
					System.exit(0);
				}
			}					
		}		

		
		ArrayList<Instrucoes> instrucoes = funcao.getInstrucoes();		
		for (int i = 0; i < instrucoes.size()-1; i++){
			Instrucoes instr = instrucoes.get(i);
			System.out.println("Aqui!");
			instr.accept(this, o);			
		}

		Identificador nomeFuncao = funcao.getId();
		Object ultimaInstrucao = instrucoes.get(instrucoes.size()-1).getChild(0);
		if (ultimaInstrucao instanceof Atribuicao){			
			Atribuicao atr = (Atribuicao) ultimaInstrucao;
			if (!atr.getId().getText().equals(nomeFuncao.getText())){
				errorReporter.reportError(nomeFuncao, "Coloque o retorno da funo");
				System.exit(0);
			} else {										
				Expressao exp = atr.getExpressao();
				TiposPrimitivos tipoExp = (TiposPrimitivos) exp.accept(this, o);
				TiposPrimitivos tipoFuncao = funcao.getTipo().getTipo();				

				if (tipoExp != tipoFuncao){
					errorReporter.reportError(nomeFuncao, "Tipo de retorno no correspondente ao declarado na funo.");
					System.exit(0);
				}
			}
		}
		else{
			errorReporter.reportError(nomeFuncao, "Coloque o retorno da funo");
			System.exit(0);
		}		
		environment.addMetodo(funcao.getId(), funcao.getListaParametros());		
		return null;		
	}
	@Override
public Object visit(Sub sub, Object o) {
		
		System.out.println("Entrei no visit de Sub!");
		ListaParamentros listaPar = sub.getListaParametros();
		Vector<Parametro> lista = listaPar.getParametros();

		for (int i = 0; i < lista.size(); i++){
			Parametro param = lista.get(i);

			Tipo tipo = (Tipo) param.getTipo();
			TiposPrimitivos tipoTipo = tipo.getTipo();
			Identificador idTemp = param.getId();

			if(!environment.containsIdentifier(idTemp)){
				environment.addIdentificador(idTemp, tipoTipo);				
			}else{
				errorReporter.reportError(idTemp, "identificador ja utilizado!");
				System.exit(0);
			}
		}

		ArrayList<Declaracao> declaracoes = sub.getDeclaracoes();		
		for (int i = 0; i < declaracoes.size(); i++){
			Declaracao decl = declaracoes.get(i);

			Tipo tipo = (Tipo) decl.getTipo();
			TiposPrimitivos tipoTipo = tipo.getTipo();
			//TiposPrimitivos tipoTipo = (TiposPrimitivos) tipo.accept(this, o);

			ListaVariavel listVar = decl.getListaVariavel();
			Vector<Identificador> listaIdent = listVar.getIdentificadores(); 

			for (int j=0; j<listaIdent.size(); j++){
				Identificador idTemp = listaIdent.elementAt(j);
				idTemp.accept(this, o);

				if(!environment.containsIdentifier(idTemp)){					
					environment.addIdentificador(idTemp, tipoTipo);					
				}else{
					errorReporter.reportError(idTemp, "identificador ja utilizado!");
					System.exit(0);
				}
			}					
		}		
		
		//OUTRAS VALIDAES:
				
		ArrayList<Instrucoes> instrucoes = sub.getInstrucoes();
		
		for (int i = 0; i < instrucoes.size(); i++){
			Instrucoes instr = instrucoes.get(i);
			instr.accept(this, o);			
		}
				
		return null;
	}

	@Override
	public Object visit(InstrucaoInput instrucaoInput, Object o) {
		Identificador id = null;
		try {
			ListaVariavel listaVar = instrucaoInput.getListaVariaveis();
			Vector<Identificador> ids = listaVar.getIdentificadores();

			for (int i = 0; i < ids.size(); i++){
				id = ids.get(i);
				if (!environment.containsIdentifier(id)){
					errorReporter.reportError(id, "Identificador " + id + " nao foi declarado.");                                
				}

			}
		} catch (ClassCastException ex){
			errorReporter.reportError(instrucaoInput, "Parmetro da instruo Input invlido.");
			System.exit(0);
		}		
		return null;
	}

	@Override
	public Object visit(InstrucaoPrint instrucaoPrint, Object o) {
		ListaExpressoes listaExp = instrucaoPrint.getListaExpressoes();
		listaExp.accept(this, o);
		return null;
	}

	@Override
	public Object visit(ListaExpressoes expressoes, Object obj) {
		ArrayList<Expressao> lista = expressoes.getExpressoes();
		if(lista.size() == 1){
			return lista.get(0).accept(this,obj);
		}else{
			for(int i= 0; i< lista.size();i++){                                
				Expressao exp = lista.get(i);
				exp.accept(this, obj);
			}

			return TiposPrimitivos.Boolean;
		}
	}

	@Override
	//public Object visit(Atribuicao atribuicao, Map<String, TiposPrimitivos> mapping, Object obj) {
		public Object visit(Atribuicao atribuicao, Object obj) {
		Identificador id = atribuicao.getId();
		//if (!environment.containsIdentifier(id) || !mapping.containsKey(id))
		if (!environment.containsIdentifier(id)){
			errorReporter.reportError(id, "identificador '" + id.getText() + "' no foi declarado -- visit (Atribuicao)");
			System.exit(0);
		} else {
			TiposPrimitivos tipoId = environment.getTipoID(id);

			Expressao exp = atribuicao.getExpressao();
			TiposPrimitivos tipoExp = (TiposPrimitivos) exp.accept(this, obj);

			if (tipoExp == TiposPrimitivos.Unknow){
				this.errorReporter.reportError(atribuicao,"A combinao de tipos no  vlida para operao.");
				System.exit(0);
			} else if (!tipoId.equals(tipoExp)){
				errorReporter.reportError(exp, "tipo diferente do declarado");				
				System.exit(0);
			}
		}

		return null;
	}

	@Override
	public Object visit(Chamada chamada, Object obj) {
		Identificador id = chamada.getId();

		if (this.environment.containsMetodo(id)){				
			ArrayList<Expressao> paramsCham = chamada.getListaParametros().getExpressoes();
			Vector<Parametro> paramsMet = environment.getListaParametros(id).getParametros();

			if (paramsCham.size() == paramsMet.size()){
				for (int i = 0; i <paramsCham.size(); i++){					
					TiposPrimitivos tipoParCha = (TiposPrimitivos) paramsCham.get(i).accept(this, obj);					
					TiposPrimitivos tipoParMet = paramsMet.get(i).getTipo().getTipo();

					if (tipoParMet!= tipoParCha){
						errorReporter.reportError(chamada, "Passagem de tipos incorreta.");
						System.exit(0);						
					}
				}				
			} else {
				errorReporter.reportError(chamada, "Quantidade de parametros incorretas.");
				System.exit(0);
			}
		} else {
			errorReporter.reportError(chamada, "Funo ou Procedimento inexistente.");
			System.exit(0);
		}
		return null;
	}

	@Override
	public Object visit(Declaracao declaracao, Object obj) {		
		Tipo tipo = declaracao.getTipo();

		ListaVariavel lista = declaracao.getListaVariavel();

		Vector<Identificador> variaveis = lista.getIdentificadores();
		Identificador id;

		for (int i = 0; i < variaveis.size(); i++){
			id = variaveis.get(i); 
			//id.accept(this, obj);

			if (!environment.containsIdentifier(id)){
				environment.addIdentificador(id, tipo.getTipo());
			} else {
				errorReporter.reportError(id, "identificador ja utilizado: " + id.getValor());
				System.exit(0);
			}
		}			

		return null;
	}


	@Override
	public Object visit(Variavel variavel, Object o) {
		return null;
	}


	@Override
	public Object visit(Expression expression, Object obj) {
		Terminal terminal = (Terminal) expression;
		if(terminal instanceof IntegerType){
			IntegerType intLit = (IntegerType) terminal;
			return intLit.accept(this, obj);
		}else if (terminal instanceof BooleanType){
			BooleanType boolLit = (BooleanType) terminal;
			return boolLit.accept(this, obj);
		} else {
			StringType boolStr = (StringType) terminal;
			return boolStr.accept(this, obj);			
		}
	}


}
